//
//  iSubAppDelegate.m
//  iSub
//
//  Created by Ben Baron on 2/27/10.
//  Copyright Ben Baron 2010. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, 
//  are permitted provided that the following conditions are met:
//  
//  * Redistributions of source code must retain the above copyright notice, this 
//    list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright notice, 
//    this list of conditions and the following disclaimer in the documentation 
//    and/or other materials provided with the distribution.
//  * Neither the my name nor the names of my contributors may be used to endorse
//    or promote products derived from this software without specific prior written 
//    permission.
//  
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
//  DAMAGE.

#import "U1MusicAppDelegate.h"
#import "ArtistListViewController.h"
#import "AlbumListViewController.h"
#import "PlaylistListViewController.h"
#import "SongListViewController.h"
#import "SettingsViewController.h"
#import "StreamingPlayer.h"
#import "Subsonic.h"
#import "UOMusicLoginController.h"
#import "URLQueryStringParser.h"

@interface U1MusicAppDelegate () <UOMusicLoginControllerDelegate>
@property (retain) UOMusicLoginController *loginController;
- (void)recoverState;
- (void)parseQueryCredentials:(NSURL*)anURL;
- (BOOL)configureUsernamePassword;
- (void)presentLoginView;
- (void)prepareTestFlight;
@end


@implementation U1MusicAppDelegate

@synthesize window;
@synthesize tabBarController;
@synthesize loginController;

#pragma mark -
#pragma mark Application lifecycle

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{   
	[self prepareTestFlight];

    [self recoverState];

    window.backgroundColor = [UIColor viewFlipsideBackgroundColor];
    [window addSubview:[tabBarController view]];

    [tabBarController setViewControllers:[NSArray arrayWithObjects:[ArtistListViewController navigableViewController], [AlbumListViewController navigableViewController], [SongListViewController navigableViewController], [PlaylistListViewController navigableViewController], [SettingsViewController navigableViewController], nil] animated:NO];

    [window makeKeyAndVisible];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(reloadCreds:)
												 name:NOTIF_reloadAccountCredentials
											   object:nil];

	if ([self configureUsernamePassword])
	{
		[self presentLoginView];
	}
	
    return YES;
}

- (void)applicationWillResignActive:(UIApplication*)application
{
	//
}

- (void)applicationDidBecomeActive:(UIApplication*)application
{
	//
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	[[StreamingPlayer sharedStreamingPlayer] saveDefaults];
}

#pragma mark -
#pragma mark Helper methods

- (void)reloadCreds:(NSNotification*)notification;
{
	[self presentLoginView];
}

- (void)parseQueryCredentials:(NSURL*)anURL
{
	URLQueryStringParser *queryStringParser = [[URLQueryStringParser alloc] initWithURL:anURL];
	NSString *user = [queryStringParser queryStringValueForKey:@"u"];
	NSString *pass = [queryStringParser queryStringValueForKey:@"p"];
	
	if (user != nil && pass != nil)
	{
		[[Subsonic sharedSubsonic] storeUsername:user password:pass];
	}
	
	[queryStringParser release];
}

- (BOOL)configureUsernamePassword
{
	BOOL needToRetrieveCredentials = NO;
	
	NSString *username = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil];
	if (username)
	{
		[[Subsonic sharedSubsonic] setDefaultUserName:username];
		[[Subsonic sharedSubsonic] setDefaultPassword:[SFHFKeychainUtils getPasswordForUsername:username andServiceName:UBUNTU_ONE_SERVICE_NAME error:nil]];
	}
	else
	{
		needToRetrieveCredentials = YES;
	}
	return needToRetrieveCredentials;
}

// Recover current state if player was interrupted
- (void)recoverState
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if ([[defaults objectForKey:@"recover"] isEqualToString:@"YES"])
	{
		[[StreamingPlayer sharedStreamingPlayer] recoverState:[[defaults objectForKey:@"isShuffle"] isEqualToString:@"YES"]];
		
		[defaults setObject:@"NO" forKey:@"recover"];
		[defaults synchronize];
	}
}

- (void)presentLoginView;
{
	self.loginController = [[[UOMusicLoginController alloc] init] autorelease];
	self.loginController.delegate = self;
	[self.tabBarController presentModalViewController:self.loginController.loginViewController animated:YES];
}

- (void)loginControllerDidLoginWithUsername:(NSString *)username;
{
	[self configureUsernamePassword];
	[self.tabBarController dismissModalViewControllerAnimated:YES];
	self.loginController = nil;
	[[NSNotificationCenter defaultCenter] postNotificationName:NOTIF_reloadAccountCredentials object:nil];
}

- (void)prepareTestFlight;
{
	NSString *tokenFilePath = [[NSBundle mainBundle] pathForResource:@"U1TestFlightTeamToken" ofType:@"txt"];
	if (!tokenFilePath)
		return;
	
	NSError *error = nil;
	NSString *teamToken = [NSString stringWithContentsOfFile:tokenFilePath usedEncoding:NULL error:&error];
	
	if (teamToken)
	{
		teamToken = [teamToken stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		[TestFlight takeOff:teamToken];
	}
	else
	{
		NSLog(@"error loading TestFlight team token: %@", error);
	}
}

#pragma mark -
#pragma mark Memory management

- (void)dealloc
{
    [tabBarController release];

    [window release];

    [super dealloc];
}

@end